#!/bin/sh

isRunning()
{
    IS_RUNNING=0
    pid=`ps -C jabberd | cut -d " " -f 2 | head -n 2`
    if [ -e jabber.pid -o -n "$pid" ]; then
        IS_RUNNING=1
    fi
}

ExitError()
{
    echo "Error: $1"
    exit 1
}

ShowWarning()
{
    warn_msg=$1

    $DIALOG --infobox "$warn_msg" 10 60
    sleep 2
}

cleanup()
{
    $DIALOG --clear
    clear
    rm -f tmp.jabberd.ans
}
    
getYesNo()
{
    title=$1
    cmnt=$2
    help_str=$3

    while /bin/true; do
        $DIALOG --backtitle "JabberD Configuration" --title "$title" --yesno "$cmnt" 10 60 2>tmp.jabberd.ans

        ret=$?
        
        if [ $ret -eq 2 ]; then
            $DIALOG --msgbox "$help_str" 10 60;
        else
            return $ret;
        fi
    done
}

getInput()
{
    title=$1
    cmnt=$2
    help_str=$3
    init=$4
    req=$5
    if [ -z "$req" ]; then
        req=0
    fi
    box_type=$6
    if [ -z "$box_type" ]; then
        box_type="inputbox"
    fi
    ret=""

    while /bin/true; do
        $DIALOG --backtitle "JabberD Configuration" --title "$title" --$box_type "$cmnt" 10 60 "$init" 2>tmp.jabberd.ans

        ret=$?
        if [ $ret -eq 2 ]; then
            $DIALOG --msgbox "$help_str" 10 60;
        elif [ $ret -eq 0 ]; then
            ret=`cat tmp.jabberd.ans`
            if [ -z "$ret" -a $req -eq 1 ]; then
                ShowWarning "This field is required"
            else
                echo > tmp.jabberd.ans
                break;
            fi
        else
            cleanup 
            return 255;
        fi;
    done
}

createUser()
{
    user=$1
    pass=$2

    if [ -z "$user" -a -z "$pass" ]; then
        ShowWarning "Invalid username or password supplied"
        return 1
    fi

    if [ -z "$cur_host" ]; then
        ShowWarning "You must either generate a configuration or select a valid hostname before editing user account information."
        return 1
    fi

    mkdir -p ./spool/$cur_host

    if [ -e "./spool/$cur_host/$user.xml" ]; then
        ShowWarning "User already exists, not creating."
        return 1
    fi

    cat > ./spool/$cur_host/$user.xml << EOF
<xdb><password xmlns='jabber:iq:auth' xdbns='jabber:iq:auth'>$pass</password><query xmlns='jabber:iq:register' xdbns='jabber:iq:register'><username>$user</username><password xmlns='jabber:iq:auth'>$pass</password></query></xdb>
EOF
    ShowWarning "User \"$user@$cur_host\" created"
}

getPrevConfig()
{
    if [ ! -e "./jabber.xml" ]; then
        return 1
    fi
    FQDN=`grep FQDN jabber.xml | cut -d "=" -f 2` || `hostname -f`
    cur_host=$FQDN
    S_NAME=`grep S_NAME jabber.xml | cut -d "=" -f 2`
    S_DESC=`grep S_DESC jabber.xml | cut -d "=" -f 2`
    S_URL=`grep S_URL jabber.xml | cut -d "=" -f 2`
    ADMIN_NAME=`grep ADMIN_NAME jabber.xml | cut -d "=" -f 2`
    USE_PID=`grep USE_PID jabber.xml | cut -d "=" -f 2`
    ALLOW_REGISTRATION=`grep ALLOW_REGISTRATION jabber.xml | cut -d "=" -f 2`
}

genConfig()
{
    if [ $IS_RUNNING = 1 ]; then
        $DIALOG --msgbox "The server is currently running. Changes you make now will not take effect until you restart the server." 10 60
    fi

    getPrevConfig

    # We don't want to do this if we don't have to
    do_regen=0

    getInput "Domain Name (REQUIRED)" "Please supply the fully qualified domain name of your Jabber server." "The domain name is a required parameter.  Fully qualified domain names are used in Jabber to verify the identity of senders and to prevent address spoofing. If your server will communicate with other servers on the Jabber network, it is important to supply the correct domain name for your server." "$FQDN" 1
    if [ $? -eq 255 ]; then 
        return 
    fi
    if [ "$FQDN" != "$ret" ]; then
        do_regen=1
        FQDN=$ret
        mkdir -p ./spool/$cur_host
    fi
    cur_host=$FQDN
    
    getInput "Server Name (OPTIONAL)" "Please supply a 'friendly name' for this server (e.g., MyCompany's Jabber Server)." "The server name is an optional parameter.  The 'friendly name' can be anything you would like. It is saved in the server's vCard and is shown to users only if they request it from the server." "$S_NAME"
    if [ $? -eq 255 ]; then 
        return 
    fi
    if [ "$S_NAME" != "$ret" ]; then
        do_regen=1
        S_NAME=$ret
    fi
    
    getInput "Server Description (OPTIONAL)" "Please supply a simple description of this server (e.g., This is the public Jabber server hosted by MyCompany)." "The server description is an optional parameter.  The server description can be anything you would like. It is saved in the server's vCard and is shown to users only if they request it from the server." "$S_DESC"
    if [ $? -eq 255 ]; then 
        return 
    fi
    if [ "$S_DESC" != "$ret" ]; then
        do_regen=1
        S_DESC=$ret
    fi

    getInput "Server URL (OPTIONAL)" "If there is a web page containing information about your Jabber server, please supply the URL." "The URL is an optional parameter. It is saved in the server's vCard and is shown to users only if they request it from the server." "$S_URL"
    if [ $? -eq 255 ]; then 
        return 
    fi
    if [ "$S_URL" != "$ret" ]; then
        do_regen=1
        S_URL=$ret
    fi

    getInput "Administrator (REQUIRED)" "Please supply a username for the server administrator." "The administrator username is required. This is the username that you will log in with to administer the server. This account will be
created for you." "$ADMIN_NAME"
    if [ $? -eq 255 ]; then 
        return 
    fi
    if [ "$ADMIN_NAME" != "$ret" ]; then
        do_regen=1
        ADMIN_NAME=$ret
    fi

    getInput "Administrator Password (REQUIRED)" "Please supply a password for the administrator." "This is the password for the previously requested administrator username.  This will be added to the created account." "" 1
    if [ $? -eq 255 ]; then 
        return 
    fi
    ADMIN_PASS=$ret
    createUser $ADMIN_NAME $ADMIN_PASS
    
    getYesNo "PID File Support" "Do you wish to enable PID file support?\n\nThis is a file that will track the process id of the Jabber server.  This can be used to easily stop the server."
    if [ "$USE_PID" != "$ret" ]; then
        do_regen=1
        USE_PID=$ret
        PIDFILE=""
        if [ $USE_PID = 0 ]; then
            PIDFILE="<pidfile>./jabber.pid</pidfile>"
        fi
    fi

    if [ $do_regen = 0 ]; then
        ShowWarning "Nothing Change, skipping regeneration"
        return 0
    fi

    regenConfig
}

regenConfig()
{
    ShowWarning "Regenerating Configuration File"

cat > jabber.xml << EOF
<!-- DO NOT EDIT - Use setup.sh to edit -->
<!-- 
Variables used:
FQDN=$FQDN
S_NAME=$S_NAME
S_DESC=$S_DESC
S_URL=$S_URL
ADMIN_NAME=$ADMIN_NAME
USE_PID=$USE_PID
ALLOW_REGISTRATION=$ALLOW_REGISTRATION
NOTIFY_REGISTER=$NOTIFY_REGISTER
-->

<jabber>
  <service id="sessions">
    <host><jabberd:cmdline flag="h">$FQDN</jabberd:cmdline></host>
    <jsm xmlns="jabber:config:jsm">
      <vCard>
        <FN>$S_NAME</FN>
        <DESC>$S_DESC</DESC>
        <URL>$S_URL</URL>
      </vCard>
EOF
if [ $ALLOW_REGISTRATION = 0 ]; then
    echo -n "      <register" >> jabber.xml
    if [ $NOTIFY_REGISTER = 0 ]; then
        echo -n " notify='yes'" >> jabber.xml
    fi
    echo ">" >> jabber.xml
cat >> jabber.xml << EOF
        <instructions>Choose a username and password to register with this server.</instructions>
        <name/>
        <email/>
      </register>
EOF
fi
cat >> jabber.xml << EOF
      <welcome>
        <subject>Welcome!</subject>
        <body>Welcome to the Jabber server at $FQDN</body>
      </welcome>
      <admin>
        <write>$ADMIN_NAME@$FQDN</write>
        <reply>
          <subject>Auto Reply</subject>
          <body>This is a special administrative address.  Your message was received and forwarded to server administrators.</body>
        </reply>
      </admin>
      <vcard2jud/>
      <browse>
        <service type="jud" jid="users.jabber.org" name="Jabber User Directory">
          <ns>jabber:iq:search</ns>
          <ns>jabber:iq:register</ns>
        </service>
      </browse>
    </jsm>
    <load main="jsm">
      <jsm>./jsm/jsm.so</jsm>
      <mod_echo>./jsm/jsm.so</mod_echo>
      <mod_roster>./jsm/jsm.so</mod_roster>
      <mod_time>./jsm/jsm.so</mod_time>
      <mod_vcard>./jsm/jsm.so</mod_vcard>
      <mod_last>./jsm/jsm.so</mod_last>
      <mod_version>./jsm/jsm.so</mod_version>
      <mod_announce>./jsm/jsm.so</mod_announce>
      <mod_agents>./jsm/jsm.so</mod_agents>
      <mod_browse>./jsm/jsm.so</mod_browse>
      <mod_admin>./jsm/jsm.so</mod_admin>
      <mod_filter>./jsm/jsm.so</mod_filter>
      <mod_offline>./jsm/jsm.so</mod_offline>
      <mod_presence>./jsm/jsm.so</mod_presence>
      <mod_auth_plain>./jsm/jsm.so</mod_auth_plain>
      <mod_auth_digest>./jsm/jsm.so</mod_auth_digest>
      <mod_auth_0k>./jsm/jsm.so</mod_auth_0k>
      <mod_log>./jsm/jsm.so</mod_log>
      <mod_register>./jsm/jsm.so</mod_register>
      <mod_xml>./jsm/jsm.so</mod_xml>
    </load>
  </service>

  <xdb id="xdb">
    <host/>
    <load>
      <xdb_file>./xdb_file/xdb_file.so</xdb_file>
    </load>
    <xdb_file xmlns="jabber:config:xdb_file">
      <spool><jabberd:cmdline flag='s'>./spool</jabberd:cmdline></spool>
    </xdb_file>
  </xdb>
  <service id="c2s">
    <load>
      <pthsock_client>./pthsock/pthsock_client.so</pthsock_client>
    </load>
    <pthcsock xmlns='jabber:config:pth-csock'>
      <authtime/>
      <karma>
        <init>10</init>
        <max>10</max>
        <inc>1</inc>
        <dec>1</dec>
        <penalty>-6</penalty>
        <restore>10</restore>
      </karma>
      <ip port="5222"/>
    </pthcsock>
  </service>

  <log id='elogger'>
    <host/>
    <logtype/>
    <format>%d: [%t] (%h): %s</format>
    <file>error.log</file>
    <!--<stderr/>-->
  </log>

  <log id='rlogger'>
    <host/>
    <logtype>record</logtype>
    <format>%d %h %s</format>
    <file>record.log</file>
  </log>
  <service id="dnsrv">
    <host/>
    <load>
      <dnsrv>./dnsrv/dnsrv.so</dnsrv>
    </load>
    <dnsrv xmlns="jabber:config:dnsrv">
    	<resend service="_jabber._tcp">s2s</resend> <!-- for supporting SRV records -->
    	<resend>s2s</resend>
    </dnsrv>
  </service>

  <service id="s2s">
    <load>
      <dialback>./dialback/dialback.so</dialback>
    </load>
    <dialback xmlns='jabber:config:dialback'>
      <legacy/>
      <!-- Use these to listen on particular addresses and/or ports.
      <ip port="7000"/>
      <ip port="5269">127.0.0.1</ip>
      -->
      <ip port="5269"/>
      <karma>
        <init>50</init>
        <max>50</max>
        <inc>4</inc>
        <dec>1</dec>
        <penalty>-5</penalty>
        <restore>50</restore>
      </karma>
    </dialback>
  </service>

  <io>
    <rate points="5" time="25"/>
  </io>

  $PIDFILE

</jabber>
EOF

    $DIALOG --msgbox "Configuration Successful!" 10 60
}

addUserDlg()
{
    getInput "New User Name" "Please supply the username of the new user." "This is the username of the user that is about to be created on the server. Include only the portion of the Jabber ID before the '@' sign (i.e., do not include the server hostname)." "" 1
    if [ $? -eq 255 ]; then
        return
    fi
    name=$ret
    getInput "New User Password" "Please supply a password for the new user." "The password for the previously entered username." "" 1
    if [ $? -eq 255 ]; then
        return
    fi
    pass=$ret
    createUser $name $pass
}

remUserDlg()
{
    if [ $IS_RUNNING = 1 ]; then
        ShowWarning "Removing a user while the server is running is not
recommended. Because JabberD caches user information, the account may not be removed if you attempt this while the server is running."
    fi

    getInput "Username to Remove" "Please supply the username of the account to be removed." "Make sure the server is not running when you remove a user.  Otherwise, the remove operation will not be successful."
    if [ $? -eq 255 ]; then
        return
    fi
    user=$ret
    if [ -z "$user" ]; then
        ShowWarning "No name supplied, nothing removed."
        return
    fi
    getPrevConfig
    if [ "$user" = $ADMIN_NAME ]; then
        ShowWarning "You may not remove the username that is configured as the administrator."
        return
    fi
    
    rm -f ./spool/$cur_host/$user.xml

    ShowWarning "User \"$user@$cur_host\" removed."
}

regUserDlg()
{
    getYesNo "Allow Public Registration" "Do you wish to allow public registration?  If you answer Yes, any user may create an account on your server."
    aret=$ret
    nret=$NOTIFY_REGISTER
    if [ "$aret" = "0" ]; then
        getYesNo "Notify Admin" "Should the server administrator(s) be notified when new users register accounts on the server?"
        nret=$ret
    fi

    if [ "$aret" != "$ALLOW_REGISTRATION" -o "$nret" != "$NOTIFY_REGISTER" ]; then
        ALLOW_REGISTRATION=$aret
        NOTIFY_REGISTER=$nret
        regenConfig
    fi
}

adminUsers()
{
    if [ $IS_RUNNING = 1 ]; then
        $DIALOG --msgbox "It is strongly suggested that you stop the server before making changes to user accounts and settings.  Because of the way JabberD caches user accounts, changes you make while the server is running may be lost." 10 60
    fi

    $DIALOG --backtitle "JabberD Configuration" --menu "Configuration Options" 12 60 5 reg "Registration Settings" add "Add a User" rem "Remove a User" 2>tmp.jabberd.ans

    ret=`cat tmp.jabberd.ans`
    case $ret in
        add)
            addUserDlg
            ;;
        rem)
            remUserDlg
            ;;
        reg)
            regUserDlg
            ;;
    esac
}

showMenu()
{
    ss=Start
    if [ $IS_RUNNING = 1 ]; then
        ss=Stop
    fi
    $DIALOG --backtitle "JabberD Configuration" --menu "Configuration Options" 12 60 5 Gen "Generate Configuration" Users "User Administration" $ss "$ss the Server" Quit "Quit Configuration" 2>tmp.jabberd.ans

    ret=`cat tmp.jabberd.ans`
    case $ret in
        Gen)
            genConfig
            ;;
        Users)
            if [ ! -e jabber.xml ]; then
                $DIALOG --msgbox "You must generate a configuration before you can administer users." 10 60
                return;
            fi
            adminUsers
            ;;
        Stop)
            killall jabberd
            ShowWarning "Stopping the server"
            ;;
        Start)
            if [ ! -e jabber.xml ]; then
                $DIALOG --msgbox "Unable to start the server, you have not yet generated a configuration." 10 60
                return;
            fi
            ./jabberd -H `pwd` -B -c jabber.xml > /dev/null 2>runerror.log
            ShowWarning "Starting the server"
            isRunning
            if [ $IS_RUNNING -eq 0 ]; then
                errlog=`cat runerror.log`
                $DIALOG --msgbox "Unable to start the server, system said:\n $errlog" 10 60
            fi
            rm runerror.log
            ;;
        Quit)
            doExit=1
            ;;
    esac
}

## Find the version of dialog to use
DIALOG=`which dialog`

if [ -z $DIALOG ]; then
    echo "No version of dialog was found on your system, aborting."
    exit 1
fi

## Initialize the file we'll use for answers somewhat securely
touch tmp.jabberd.ans
chmod og-rwx tmp.jabberd.ans

## Last bit of setup for initial vars
getPrevConfig

doExit=0
while [ $doExit = 0 ]
do
    isRunning
    showMenu;
done

cleanup
